# this file is generated by Lino
from django.utils.translation import ugettext
ugettext('Next page')
ugettext('Please wait...')
ugettext('Show or hide the table parameters panel')
ugettext('Action not available on phantom record.')
ugettext('Enter a date in format YYYY-MM-DD (use zeroes for unknown parts).')
ugettext('Next')
ugettext('Go to record')
ugettext('Reload current record')
ugettext('Save changes to current record ?')
ugettext('Edit text in own window')
ugettext('Enter a text to use as quick search filter')
ugettext('Username')
ugettext('Confirmation')
ugettext('Last page')
ugettext('Alert')
ugettext('Previous page')
ugettext('Cannot show detail for empty foreign key.')
ugettext('Quick Search')
ugettext('This record is disabled')
ugettext('Password')
ugettext('Displaying {0} - {1} of {2}')
ugettext('Team view')
ugettext('Last')
ugettext('Edit help texts for fields on this model.')
ugettext('Show this table in plain html')
ugettext('Export this table to a .csv file')
ugettext('of {0}')
ugettext('Log in')
ugettext('Save Grid Configuration')
ugettext('Page')
ugettext('First')
ugettext('Show this panel in own window')
ugettext('This field is disabled')
ugettext('Delete {0} rows. Are you sure?')
ugettext('No more records to display. Detail window has been closed.')
ugettext('First page')
ugettext('Show this table as a pdf document')
ugettext('Select another view of this report')
ugettext('Cannot show MTI child if checkbox is off.')
ugettext('View')
ugettext('Save changes to text ?')
ugettext('Previous')
