<div class="htmlText">
{% if not request.user.is_authenticated %}
{% if not site.admin_prefix %}
<p>
{{_("Welcome to the <b>%s</b> site.") % site.title}}.
We are running 
<a href="{{site.url}}">{{site.short_name}}</a>
version {{site.version}}, {{site.description}}
</p>
{% endif %}

<p>
{{_("You have entered the <b>admin</b> section.")}}
{{_("Unlike the <a href=\"/\">web content section</a> there is now a GUI menu bar in the upper part of the screen.")}}
</p><p>
{{_("You will now probably want to use the <b>Log in</b> button in the upper right corner and log in.")}}
</p><p>
{{_("This demo site has %d users, they all have \"1234\" as password:") % site.modules.users.UsersOverview.request().get_total_count()}}

{{as_ul('users.UsersOverview')}}

<p>
{{_("Your feedback is welcome to lino-users@googlegroups.com or directly to the person who invited you.")}}
</p>
{% endif %}
</div>