/*
 Copyright 2009-2012 Luc Saffre
 This file is part of the Lino project.
 Lino is free software; you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 Lino is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Lino; if not, see <http://www.gnu.org/licenses/>.
*/

/* 
http://stackoverflow.com/questions/2106104/word-wrap-grid-cells-in-ext-js 
*/
.x-grid3-cell-inner {
  /* white-space: normal; changed from nowrap */
  height:18px; overflow:hidden;
}
.x-grid3-hd-inner {
  white-space: normal; 
}

/*.lino-phantom-row table.x-grid3-row-table {
	color: blue; 
}
*/
.lino-phantom-row td {
	/* font-style: italic; */
	color:white;
}

/* 
.x-item-disabled, .x-tree-node-disabled, .x-date-disabled
*/
.x-item-disabled {
  color: blue; 
  /*   opacity: 1;  
  font-weight: bold; */
}

.x-item-disabled * {
  color: blue !important; 
  /*font-weight: bold !important; */
}

.x-btn-noicon {
	color: red; font-style:italic;
}
.x-btn-pressed {
	color: red; font-style:italic;
}

.x-btn-text {
	color: red; font-style:italic;
}

.x-btn-menu-active {
	cursor: 'wait';
	color:'red';
}

/* See blog/2011/0603
*/
.x-form-item-label{
    white-space: nowrap; 
}




/* 
.x-form-field { padding: 10px; }
*/

.x-panel-body-noheader { padding-left: 2px; padding-right: 2px; }
.x-panel-body { padding-left: 2px; padding-right: 2px; }



/* TinyMCE specific rules */
/* 
body.mceContentBody {
  color: red !important; 
}
*/
body.mceNonEditable {
  color: blue !important; 
}

/*

To display "normal" HTML inside an ExtJS component, 
wrap it into a <div class="htmlText">.

*/

.htmlText {
	font-family: arial,tahoma;
}
.htmlText h1 {
	font-size:16px;
	padding-top:0px;
	padding-bottom:12px;
	}
.htmlText h2 {
	font-size:13px;
	padding-top:0px;
	padding-bottom:10px;
	}
.htmlText td {
	padding:0px;
	}
.htmlText p {
	margin-top:0px;
	margin-bottom:0px;
	}
.htmlText ol {
	margin-top:3px;
	margin-left:20px;
	margin-bottom:3px;
	list-style-type: decimal; 
	list-style-image: none; 
	list-style-position: outside;
	}
.htmlText ul {
	margin-top:3px;
	margin-left:20px;
	margin-bottom:3px;
	list-style-type: square; 
	list-style-image: none; 
	list-style-position: outside;
	}
.htmlText strong {
	font-weight:bold;
	}
.htmlText em {
	font-style: italic;
	}

/* 
Thanks to http://de.selfhtml.org/css/formate/zentrale.htm
*/
table.htmlText tr td {
	vertical-align: middle;
	text-align: center;
	}



/* 
The following are 
Alexander Dawson's CSS tooltips
from <http://sixrevisions.com/css/css-only-tooltips/>,
adapted for usage in an ExtJS application.

*/

.x-form-item-label .tooltip {
	border-bottom: 1px dotted #000000; color: #000000; outline: none;
	cursor: help; 
	text-decoration: none;
	position: relative;
}
.x-form-item-label .tooltip span {
	margin-left: -999em;
	position: absolute;
}
.x-form-item-label .tooltip:hover span {
	border-radius: 5px 5px; -moz-border-radius: 5px; -webkit-border-radius: 5px; 
	box-shadow: 5px 5px 5px rgba(0, 0, 0, 0.1); -webkit-box-shadow: 5px 5px rgba(0, 0, 0, 0.1); 
	-moz-box-shadow: 5px 5px rgba(0, 0, 0, 0.1);
	font-family: Calibri, Tahoma, Geneva, sans-serif;
	position: absolute; left: 1em; top: 2em; z-index: 99;
	overflow: visible;
    white-space: normal; 
	margin-left: 0; width: 250px;
	
}
.x-form-item-label .tooltip:hover img {
	border: 0; margin: -10px 0 0 -55px;
	float: left; position: absolute;
}
.x-form-item-label .tooltip:hover em {
	font-family: Candara, Tahoma, Geneva, sans-serif; font-size: 1.2em; font-weight: bold;
	display: block; padding: 0.2em 0 0.6em 0;
}
.x-form-item-label .classic { padding: 0.8em 1em; }
.x-form-item-label .custom { padding: 0.5em 0.8em 0.8em 2em; }
.x-form-item-label * html a:hover { background: transparent; }
.x-form-item-label .classic {background: #FFFFAA; border: 1px solid #FFAD33; }
.x-form-item-label .critical { background: #FFCCAA; border: 1px solid #FF3334;	}
.x-form-item-label .help { background: #9FDAEE; border: 1px solid #2BB0D7;	}
.x-form-item-label .info { background: #9FDAEE; border: 1px solid #2BB0D7;	}
.x-form-item-label .warning { background: #FFFFAA; border: 1px solid #FFAD33; }


/*
 20120229 For css tooltips we need to override some CSS rules of ExtJS.
 set all overflows 
 to 'visible' for CSS tooltips.
 I don't really understand it, and it doesn't yet work,
 but it doesn't seem to  cause any problem.
 
.ext-gecko .x-window-body .x-form-item {
	overflow: hidden;
}

*/

.ext-gecko .x-window-body .x-form-item {
	overflow: visible;
}
.x-panel-body .x-panel-body-noheader {
	overflow: visible;
}
.x-panel-bwrap {
	overflow: visible;
}

.x-box-inner {
	overflow: visible;
}
.x-fieldset .x-form-label-top .x-box-item {
	overflow: visible;
}


/* 
.x-tip 
  .x-tip-tl
    .x-tip-tr
      .x-tip-tc
        .x-tip-header
          .x-tip-header-text
  .x-tip-bwrap
    .x-tip-ml
      .x-tip-mr
        .x-tip-mc
          .x-tip-body
.x-tip-bwrap div {
	background: #FFFFAA; 
}	
.x-tip-header-text { border: 0; }

.x-tip div {
	background: yellow; 
}	
.x-tip-bwrap div {
	background: yellow; 
}	
.x-tip-ml div {
	background: yellow; 
}	
.x-tip-mr div {
	background: yellow; 
}	
.x-tip-mc div {
	background: yellow; 
}	
.x-tip-body div {
	background: yellow; 
}	
*/

/* border: 1px solid #FFAD33;  */


.x-tbar-delete{
	background-image: url(images/mjames/delete.png) !important;
}
/*
.x-item-disabled .x-tbar-delete{
	background-image: url(images/default/grid/delete-disabled.gif) !important;
}
*/

.x-tbar-new{
	background-image: url(images/mjames/add.png) !important;
}

/*
ShowDetailAction
$a.icon_name
$a.icon_file
*/
.x-tbar-detail{
	background-image: url(images/mjames/application_form.png) !important;
}

.x-tbar-save{
	background-image: url(images/mjames/disk.png) !important;
}

.x-tbar-email_add{
	background-image: url(images/mjames/email_add.png) !important;
}
.x-tbar-email_send{
	background-image: url(images/mjames/email_go.png) !important;
}

.x-tbar-create-postings{
	background-image: url(images/mjames/script_add.png) !important;
}

.x-tbar-calendar{
	background-image: url(images/mjames/calendar.png) !important;
}

.x-tbar-print{
	background-image: url(images/mjames/printer.png) !important;
}

.x-tbar-clearcache{
	background-image: url(images/mjames/printer_delete.png) !important;
}
.x-tbar-duplicate{
	background-image: url(images/mjames/arrow_divide.png) !important;
}
.x-tbar-pdf{
	background-image: url(images/mjames/page_white_acrobat.png) !important;
}
.x-tbar-html{
	background-image: url(images/mjames/html.png) !important;
}
.x-tbar-csv{
	background-image: url(images/mjames/page_excel.png) !important;
}
.x-tbar-vcard{
	background-image: url(images/mjames/vcard.png) !important;
}
.x-tbar-done{
	background-image: url(images/mjames/accept.png) !important;
}
.x-tbar-parameters{
	background-image: url(images/mjames/database_gear.png) !important;
}
