{% extends "voc/base_site.html" %}

{% block title %}{{ object.pos }}{% endblock %}

{% block content %}

<div id="breadcrumbs"><a href="{{ root_url }}">Home</a> / <a href="{{ model.url }}">{{ model.verbose_name_plural|capfirst }}</a> / <a href="{{ field.url }}">By {{ field.field.verbose_name }}</a> / {{ value }}</div>

<h1>{{ object.question }}</h1>

<a href="{{ unit.get_absolute_url }}">{{ unit }}</a>

<h1>{{ object.answer }}</h1>

{% endblock %}
